# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################
"""Module to manage submissions"""
import copy
import dateutil.parser
import json
import scoring

from datetime import datetime, date
from dateutil.relativedelta import *
from dateutil import tz

import numpy as np
import pylab

from dreamtools.core.sageutils import Login

__all__  = ["SubmissionTools", "SC1ASubmissions", "SC1BSubmissions",
    "SC2BSubmissions", "SC2BSubmissions"]


class SubmissionTools(Login):
    """Utilities to manipulate list of submissions.

    .. note:: Used in hpn module and SCXYSubmissions classes but users
        do not need this module. For book keeping only.

    .. seealso:: ranking module

    This class could be used to attach information to submissions, load the
    status from synapse and attached it to the submissions, plot summary,
    and as a base class for the different sub challenges.::

        h = HPN.Admin()
        submissions = h.get_submissions_network( status="all")

        subs = hpn.SubmissionTools()
        submissions = subs.attach_status_to_submissions(submissions)
        submissions = subs.attach_week_to_submissions(submissions)

        [(sub['submitterAlias'], sub['week'], sub['createdOn'],
            sub['substatus']['status'], sub['substatus']['score']) for sub in
            submissions]

    """
    def __init__(self, client=None, username=None, password=None, name="notset"):
        super(SubmissionTools, self).__init__(client=client)
        self.sc1a_weeks = {
            1: ("2013-07-15T08:00:00.000Z", "2013-07-30T08:00:00.000Z"),
            2: ("2013-07-30T08:00:00.000Z", "2013-08-06T11:00:00.000Z"),  # was published using 11am
            3: ("2013-08-06T11:00:00.000Z", "2013-08-13T08:00:00.000Z"),
            4: ("2013-08-13T08:00:00.000Z", "2013-08-20T08:00:00.000Z"),
            5: ("2013-08-20T08:00:00.000Z", "2013-08-27T08:00:00.000Z"),
            6: ("2013-08-27T08:00:00.000Z", "2013-09-03T08:00:00.000Z"),
            7: ("2013-09-03T08:00:00.000Z", "2013-09-10T08:00:00.000Z"),
            8: ("2013-09-10T08:00:00.000Z", "2013-09-17T08:00:00.000Z"),
            9: ("2013-09-17T08:00:00.000Z", "2013-09-23T08:00:00.000Z"),
            10: ("2013-09-23T08:00:00.000Z", "2013-09-30T08:00:00.000Z"),
            "collaborative": ("2013-11-01T08:00:00.000Z", "2014-01-31T08:00:00.000Z")
        }

        self.sc1b_weeks = {
            1: ("2013-07-15T08:00:00.000Z", "2013-07-30T08:00:00.000Z"),
            2: ("2013-07-30T08:00:00.000Z", "2013-08-06T08:00:00.000Z"),
            3: ("2013-08-06T08:00:00.000Z", "2013-08-13T08:00:00.000Z"),
            4: ("2013-08-13T08:00:00.000Z", "2013-08-20T08:00:00.000Z"),
            5: ("2013-08-20T08:00:00.000Z", "2013-08-27T08:00:00.000Z"),
            6: ("2013-08-27T08:00:00.000Z", "2013-09-03T08:00:00.000Z"),
            7: ("2013-09-03T08:00:00.000Z", "2013-09-10T08:00:00.000Z"),
            8: ("2013-09-10T08:00:00.000Z", "2013-09-17T08:00:00.000Z"),
            9: ("2013-09-17T08:00:00.000Z", "2013-09-23T08:00:00.000Z"),
            10: ("2013-09-23T08:00:00.000Z", "2013-09-30T08:00:00.000Z"),
            "collaborative": ("2013-11-01T08:00:00.000Z", "2014-01-31T08:00:00.000Z")
        }

        self.sc2a_weeks = {
            1: ("2013-07-15T08:00:00.000Z", "2013-08-13T08:00:00.000Z"),
            2: ("2013-08-13T08:00:00.000Z", "2013-08-20T08:00:00.000Z"),
            3: ("2013-08-20T08:00:00.000Z", "2013-08-27T08:00:00.000Z"),
            4: ("2013-08-27T08:00:00.000Z", "2013-09-03T08:00:00.000Z"),
            5: ("2013-09-03T08:00:00.000Z", "2013-09-10T08:00:00.000Z"),
            6: ("2013-09-10T08:00:00.000Z", "2013-09-17T08:00:00.000Z"),
            7: ("2013-09-17T08:00:00.000Z", "2013-09-23T08:00:00.000Z"),
            8: ("2013-09-23T08:00:00.000Z", "2013-09-30T08:00:00.000Z"),
            "collaborative": ("2013-11-01T08:00:00.000Z", "2014-01-31T08:00:00.000Z")
        }

        self.sc2b_weeks = {
            1: ("2013-07-15T08:00:00.000Z", "2013-08-20T08:00:00.000Z"),
            2: ("2013-08-20T08:00:00.000Z", "2013-08-27T08:00:00.000Z"),
            3: ("2013-08-27T08:00:00.000Z", "2013-09-03T08:00:00.000Z"),
            4: ("2013-09-03T08:00:00.000Z", "2013-09-10T08:00:00.000Z"),
            5: ("2013-09-10T08:00:00.000Z", "2013-09-17T08:00:00.000Z"),
            6: ("2013-09-17T08:00:00.000Z", "2013-09-23T08:00:00.000Z"),
            7: ("2013-09-23T08:00:00.000Z", "2013-09-30T08:00:00.000Z"),
            "collaborative": ("2013-11-01T08:00:00.000Z", "2014-01-31T08:00:00.000Z")
        }
        self.name = name
        self.submissions = []

    def attach_week_to_submissions(self, submissions , challenge):
        for i,submission in enumerate(submissions):
            week = self.get_week_submission(submission, challenge=challenge)
            submission['week'] = week
        return submissions

    def _keep_latest_only(self):
        submitters = self.get_unique_submitters()
        latest = []
        for submitter in submitters:
            subs = [x for x in self.submissions if x['userId'] == submitter]
            latest.append(subs[-1])
        return latest

    def get_week_submission(self, submission, challenge=None):
        assert challenge in ["sc1a", "sc1b", "sc2a", "sc2b"]
        d = dateutil.parser.parse(submission['createdOn'])

        if challenge == "sc1a":
            weeks = self.sc1a_weeks
        elif challenge == "sc1b":
            weeks = self.sc1b_weeks
        elif challenge == "sc2a":
            weeks = self.sc2a_weeks
        elif challenge == "sc2b":
            weeks = self.sc2b_weeks
        else:
            raise NotImplementedError

        result = None
        for week in sorted(weeks.keys()):
            dstart = dateutil.parser.parse(weeks[week][0])
            dend = dateutil.parser.parse(weeks[week][1])
            if d >= dstart and d<=dend:
                if result !=None:
                    raise ValueError("found submission in 2 weeks ???")
                else:
                    result = week

        if result == None: # check before ?
            dstart = dateutil.parser.parse(weeks[1][0])
            if d <= dstart:
                result = 0
        return result

    def attach_status_to_submissions(self, submissions):
        for i,submission in enumerate(submissions):
            print("attaching status to sub %s" % i)
            substatus = self.client.getSubmissionStatus(submission['id'])
            submission['substatus'] = copy.deepcopy(substatus)
        return submissions

    def summary(self, show="all"):
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])
        for i in ranks:
            sub = self.submissions[i]
            print("%10s | %5s |%5s |%s|  %s |%s" % (sub['userId'], sub['submitterAlias'],sub['week'],
                sub['substatus']['status'], sub['ranking'],sub['zscore']))

    def attach_user_profile(self, submissions):
        for i, submission in enumerate(submissions):
            print("fetching profile of user %s" % i)
            profile = self.client.getUserProfile(submission['userId'])
            submission['userProfile'] = profile.copy()
        return submissions

    def get_unique_submitters(self):
        return list(set([x['userId'] for x in self.submissions]))


class SC1ASubmissions(SubmissionTools):
    """Retrieve SCORED submissions and attach all relevant information

    >>> s = SC1ASubmissions(final=True)
    >>> s.load_submissions()
    >>> len(s.submissions)


    final = True means that 4 combi of cell line/ligands will be ignored. See
    code

    """
    def __init__(self, client=None, name="SC1A", final=True):
        super(SC1ASubmissions, self).__init__(client=client, name=name)
        from hpn import HPNAdmin
        self.hpn = HPNAdmin(client=self.client)
        self.final = final

    def load_submissions(self, startweek=0, endweek=9, keep_latest=True):
        """Loads all SCORED submissions from SC1A

        Attaches the week, status, ranking and zscores

        """
        # load all scored submissions
        self.submissions = self.hpn.get_submissions_network(status="SCORED")
        print("Got %s SCORED submissions" % len(self.submissions))

        # attach week and filter the submissions
        self.submissions = self.attach_week_to_submissions(self.submissions, "sc1a")
        self.submissions = [sub for sub in self.submissions if sub['week']<=endweek]
        self.submissions = [sub for sub in self.submissions if sub['week']>=startweek]
        print("Keeping %s submissions in the week range requested" % len(self.submissions))

        if keep_latest:
            self.submissions = self._keep_latest_only()
            print("Keep %s latest scored submissions " % len(self.submissions))
        #assert len(self.submissions) == 83, "There were 83 submissions valid in  SC1A"

        # attach the zscore
        print("attaching submissions")
        self.submissions = self.attach_status_to_submissions(self.submissions)

        print("remove some users")
        self.remove_users()

        print("attaching scores and compute final ranking")
        self._attach_zscores() #  attach zscores
        print("all submissions available in the **submissions** attribute")
        #return self.submissions

    def remove_users(self, userIds=["375805", "1971259", "1991105", "1961142",
    "2208193", "2154231", "2200202", "2197351", "375570"]):
        """Remove some users to get final submissions

            * 375805 alphabeta is a test from TC
            * sakevin is the same team as sakev Last submission by sakve in on week
              8. sakevin submitted last on week 2 so can be ignored (userId 1991105)
            * hd systems submitted with 2 userId:
              [(u'1967990', u'HD_Systems', u'Michael Zengerling', 8)]
              [(u'1971259', u'HD_Systems', u'Ruth Grosseholz', 7)]
              The latest has the best score (week 8) so I suspect this is the one to be
              used.
            * SHCH and tongki aliases correspond to the same userId. tongji
              submitted last

        From Steven's analysis, some results are highly correlated:

            * ams1012,cas3,cas4 are the same submission. We keep only ams1012
              cas3 1961142, cas4 2208193 should be removed.
            * gucas (2154231) is same as Zhangroup. gucas removed
            * Dream5607 (2200202) and Pitt.transmed are the same. Remove Dream5607
            * sfntt has same alias but 2 different userId. Remove 2197351 so that
              the latest remains only.
            * same with SBIT 375570


        """
        submissions = [x for x in self.submissions if x['userId'] not in userIds]
        self.submissions = submissions

    def _get_ranking(self):
        ranking = scoring.HPNScoringNetwork_ranking()
        for i,sub in enumerate(self.submissions):
            auc = json.loads(sub['substatus']['report'])
            ranking.add_auc(auc, sub['submitterAlias'] +"_"+ str(i))
        return ranking

    def _attach_zscores(self):
        """attach mean zscore, 32 individual zscores
        """
        ranking = self._get_ranking()
        zscores = ranking.get_mean_zscores()

        # need to remove the 4 cell lines that causes trouble. cannot be done in
        # ranking function

        ranks = ranking.get_mean_ranks()
        for i,participant in enumerate(ranking.participants):
            # mean zscore
            self.submissions[i]['zscore'] = zscores[participant]
            # 32 individual zscores
            self.submissions[i]['zscores'] = ranking._get_zscores(ranking.aucs[i])
            # final rank
            self.submissions[i]['ranking'] = ranks[participant]
            # 32 individual ranks
            self.submissions[i]['ranks'] =  ranking.get_rank_participant(participant)
            # 32 individual AUCS
            self.submissions[i]['aucs'] =  ranking.aucs[i]
            this = ranking.aucs[i]
            self.submissions[i]['mean_aucs'] =  np.mean([this[k1][k2] for k1 in this.keys()
                for k2 in this[k1].keys()])

    def summary_final(self, show="all"):
        print("Remove 3 combi of cell line/ligands before printing")
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])
        header = ("Final Rank", "Team name", "Team Id", "Submission Id", "Entity Id", "mean AUC", "mean Rank")
        print("| %12s | %20s | %20s | %20s| %20s | %12s  | %12s | " %  header)
        print("|%s|%s|%s|%s|%s|%s|%s|" % ("-"*12, "-"*20, "-"*12, "-"*12, "-"*12,"-"*12, "-"*12))

        results = {}
        res = {}
        for count, i in enumerate(ranks):
            sub = self.submissions[i]
            pvalue = 0
            if sub['submitterAlias'] == "ChaosLab":
                sub['submitterAlias'] = "FreiburgBiossX"
            data = (count+1, sub['submitterAlias'], sub['userId'],
                sub['substatus']['id'], sub['substatus']['entityId'],
                sub['mean_aucs'],sub['ranking'], pvalue)
            print("|%12s | %20s | %12s |%12s| %12s | %10.6s |%12.6s|%s|" % data)
            res[sub['submitterAlias']] = count+1
            results[sub['submitterAlias']] = data[:]
        return results


class SC1BSubmissions(SubmissionTools):
    def __init__(self, client=None, name="SC1B"):
        super(SC1BSubmissions, self).__init__(client=client, name=name)
        from hpn import HPNAdmin
        self.hpn = HPNAdmin(client=self.client)

    def load_submissions(self, startweek=0, endweek=9, keep_latest=True):
        """Loads all SCORED submissions from SC1B

        Attaches the week, status, ranking and zscores

        """
        # load all scored submissions
        self.submissions = self.hpn.get_submissions_network_insilico(status="SCORED")
        print("Got %s SCORED submissions" % len(self.submissions))

        # attach week and filter the submissions
        self.submissions = self.attach_week_to_submissions(self.submissions, "sc1b")
        self.submissions = [sub for sub in self.submissions if sub['week']<=endweek]
        self.submissions = [sub for sub in self.submissions if sub['week']>=startweek]
        print("Keeping %s submissions in the week range requested" % len(self.submissions))

        if keep_latest:
            self.submissions = self._keep_latest_only()
            print("Keep %s latest scored submissions " % len(self.submissions))

        # attach the zscore
        print("attaching submissions")
        self.submissions = self.attach_status_to_submissions(self.submissions)

        print("remove soem users")
        self.remove_users()

        print("attaching scores and compute final ranking")
        self._attach_zscores() #  attach zscores
        print("all submissions available in the **submissions** attribute")
        #return self.submissions

    def remove_users(self, userIds=["375805", "1971259", "1961142", "2208193",
        "2154231", "2023612", "2200202"]):
        """

        * 375805 alphabeta is a test from TC
        * 1971259 HD_systems has 2 ID see SC1A remove_users function.

        # From Steven's analysis, some results are highly correlated:

        * ams1012,cas3,cas4 are the same submission. We keep only ams1012
          cas3 1961142, cas4 2208193 should be removed.
        * gucas (2154231) is same as Zhangroup. gucas removed
        * remove chilin 2023612
        * Dream5607 (2200202) and Pitt.transmed are the same. Remove Dream5607
        * sfntt has same alias but 2 different userId. Remove 2197351 so that
          the latest remains only.

        """
        submissions = [x for x in self.submissions if x['userId'] not in userIds]
        self.submissions = submissions

    def _attach_zscores(self):
        """attach mean zscore, 32 individual zscores
        """

        aucs = []
        for i, submission in enumerate(self.submissions):
            report = json.loads(submission['substatus']['report'])
            auc = report['auc']
            aucs.append(auc)
        ranks = np.argsort(aucs)[::-1] # sort a
        for i, rank in enumerate(ranks):
            self.submissions[rank]['ranking'] = i
            report = json.loads(self.submissions[i]['substatus']['report'])
            self.submissions[i]['auc'] = report['auc']
            self.submissions[i]['zscore'] = report['score']

    def summary_final(self):

        header = ("Final rank", "Team name", "userID", "synapse ID", "entityID" , "AUC", "zscore", "p-value")
        print("| %12s | %20s | %20s | %20s | %12s  | %12s | %12s | %12s |" %  header)
        print("|%s|%s|%s|%s|%s|%s|" % ("-"*12, "-"*20, "-"*12, "-"*12,"-"*12, "-"*12))
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])
        results = {}
        for count, i in enumerate(ranks):
            sub = self.submissions[i]
            pvalue = 0
            if sub['submitterAlias'] == "ChaosLab":
                sub['submitterAlias'] = "FreiburgBiossX"


            data = (count+1, sub['submitterAlias'], sub['userId'],
                        sub['substatus']['id'], sub['substatus']['entityId'],
                    sub['auc'], sub['zscore'], pvalue)
            print("|%12s | %20s | %20s | %20s | %10.6s|  %10.6s |%10.6s |%12.6s|" % data)
            results[sub['submitterAlias']] = data[:]
        return results


class SC2ASubmissions(SubmissionTools):
    def __init__(self, client=None, name="SC2A"):
        super(SC2ASubmissions, self).__init__(client=client)
        # should be here to avoid import cycling
        from hpn import HPNAdmin
        self.hpn = HPNAdmin(client=self.client)

    def load_submissions(self, startweek=0, endweek=9, keep_latest=True):
        """Loads all SCORED submissions from SC2A

        Attaches the week, status, ranking and zscores

        """
        # load all scored submissions
        self.submissions = self.hpn.get_submissions_prediction(status="SCORED")
        print("Got %s SCORED submissions" % len(self.submissions))

        # attach week and filter the submissions
        self.submissions = self.attach_week_to_submissions(self.submissions,"sc2a")
        self.submissions = [sub for sub in self.submissions if sub['week']<=endweek]
        self.submissions = [sub for sub in self.submissions if sub['week']>=startweek]
        print("Keeping %s submissions in the week range requested" % len(self.submissions))

        if keep_latest:
            self.submissions = self._keep_latest_only()
            print("Keep %s latest scored submissions " % len(self.submissions))

        # attach the zscore
        print("attaching submissions")
        self.submissions = self.attach_status_to_submissions(self.submissions)

        print("remove some users")
        self.remove_users()

        print("attaching scores and compute final ranking")
        self._attach_zscores() #  attach zscores
        print("all submissions available in the **submissions** attribute")

        for sub in self.submissions:
            rmse = json.loads(sub['substatus']['report'])
            sub['old_rmses'] = copy.deepcopy(rmse)

    def remove_users(self, userIds=["375805", "1991105", "1971259"]):
        """

        * 375805 alphabeta is a test from TC
        * 1991105 sakve from week 5 has different id from sakev week 6. renmove
          week5 that has a lower score anyway
        * HD systems see SC1A function docstring 1971259

        """
        submissions = [x for x in self.submissions if x['userId'] not in userIds]
        self.submissions = submissions

    def _get_ranking(self):
        print("Getting ranking")
        import json
        ranking = scoring.HPNScoringPrediction_ranking()
        for i,sub in enumerate(self.submissions):
            rmse = json.loads(sub['substatus']['report'])
            filename = self.client.getSubmission(sub, downloadFile=True, ifcollision="keep.local")['filePath']
            print(i, filename)
            s = scoring.HPNScoringPrediction(filename)
            s.compute_all_rmse()
            rmse = copy.deepcopy(s.rmse)

            ranking.add_rmse(rmse, sub['submitterAlias'] +"_"+ str(i))
        return ranking

    def get_final_pvalue(self, submission):
        from scipy import stats
        # get all zscores
        zz = submission['zscores']
        zscores = [zz[k1][k2] for k1 in zz.keys() for k2 in zz[k1].keys()]
        dof = len(zscores) * 2
        assert dof == 56

        # zscores are one-sided (could negative) so multiply by 1
        sided = 1
        total_score = sum([-2 * np.log(stats.norm.sf(x) * sided) for x in zscores])
        # this is a fisher method to combine the 32 scores.
        # chi2 survival for dof=64 and x=100 is 0.002686
        pvalue = stats.chi2.sf(total_score, dof)
        return pvalue

    def _attach_zscores(self):
        """attach mean zscore, 32 individual zscores
        """
        ranking = self._get_ranking()
        zscores = ranking.get_mean_zscores()
        ranks = ranking.get_mean_ranks()

        for i,participant in enumerate(ranking.participants):
            # mean zscore
            self.submissions[i]['zscore'] = zscores[participant]
            # 32 individual zscores
            self.submissions[i]['zscores'] = ranking._get_zscores(ranking.rmse[i])
            # final rank
            self.submissions[i]['ranking'] = ranks[participant]
            # 32 individual ranks
            self.submissions[i]['ranks'] =  ranking.get_rank_participant(participant)
            # 32 individual RMSEs
            self.submissions[i]['rmses'] =  ranking.rmse[i].copy()
            # 32 individual RMSEs
            s = ranking.rmse[i]
            data =  [s[k1][k2] for k1 in s.keys() for k2 in s[k1].keys()]
            data = [x for x in data if np.isnan(x)==False]
            mu = np.mean(data)
            self.submissions[i]['mean_rmse'] =  mu

    def summary_final(self):
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])

        header = ("| Final rank| Team name | Team Id | Synapse ID |  Entity ID | mean RMSE  | mean Rank | mean zscore |")
        print(header)
        print("|--------|--------|----------|-------|----|---------------|")
        results = {}
        for i, rank  in enumerate(ranks):
            sub = self.submissions[rank]
            print("|%s | %20s | %20s | %20s | %20s|  %10.6s  | %10.6s |%10.6s |" %
                    (i+1, sub['submitterAlias'],sub['userId'],
                        sub['substatus']['id'], sub['substatus']['entityId'],sub['mean_rmse'],sub['ranking'],sub['zscore']))
            results[sub['submitterAlias']] =  i+1
        return results


class SC2BSubmissions(SubmissionTools):
    def __init__(self, client=None):
        super(SC2BSubmissions, self).__init__(client=client, name="SC2B")
        from hpn import HPNAdmin
        self.hpn = HPNAdmin(client=self.client)

    def load_submissions(self, startweek=0, endweek=9, keep_latest=True):
        """Loads all SCORED submissions from SC2A

        Attaches the week, status, ranking and zscores

        """
        # load all scored submissions
        self.submissions = self.hpn.get_submissions_prediction_insilico(status="SCORED")
        print("Got %s SCORED submissions" % len(self.submissions))

        # attach week and filter the submissions
        self.submissions = self.attach_week_to_submissions(self.submissions,"sc2b")
        self.submissions = [sub for sub in self.submissions if sub['week']<=endweek]
        self.submissions = [sub for sub in self.submissions if sub['week']>=startweek]
        print("Keeping %s submissions in the week range requested" % len(self.submissions))

        if keep_latest:
            self.submissions = self._keep_latest_only()
            print("Keep %s latest scored submissions " % len(self.submissions))

        # attach the zscore
        print("attaching submissions")
        self.submissions = self.attach_status_to_submissions(self.submissions)

        print("remove soem users")
        self.remove_users()

        print("attaching scores and compute final ranking")
        self._attach_zscores() #  attach zscores
        print("all submissions available in the **submissions** attribute")
        for sub in self.submissions:
            rmse = json.loads(sub['substatus']['report'])
            sub['old_rmses'] = copy.deepcopy(rmse)

    #def remove_users(self, userIds=["375805", "1991105", "1971259"]):
    def remove_users(self, userIds=["375805", "1991105"]):
        """

        * 375805 alphabeta is a test from TC
        * 1991105 sakve from week 5 has different id from sakev week 6. renmove
          week5 that has a lower score anyway
        * HD systems see SC1A function docstring 1971259

        """
        submissions = [x for x in self.submissions if x['userId'] not in userIds]
        self.submissions = submissions

    def _get_ranking(self):
        import json
        ranking = scoring.HPNScoringPredictionInsilico_ranking()
        for i,sub in enumerate(self.submissions):
            rmse = json.loads(sub['substatus']['report'])
            filename = self.client.getSubmission(sub, downloadFile=True, ifcollision="keep.local")['filePath']
            s = scoring.HPNScoringPredictionInsilico(filename)
            s.compute_all_rmse()
            rmse = copy.deepcopy(s.rmse)
            ranking.add_rmse(rmse, sub['submitterAlias'] +"_"+ str(i))
        return ranking

    def _attach_zscores(self):
        """attach mean zscore, 32 individual zscores
        """
        ranking = self._get_ranking()
        zscores = ranking.get_mean_zscores()
        ranks = ranking.get_mean_ranks()

        for i,participant in enumerate(ranking.participants):
            # mean zscore
            self.submissions[i]['zscore'] = zscores[participant]
            # final rank
            self.submissions[i]['ranking'] = ranks[participant]
            rmse = ranking.rmse[i]
            self.submissions[i]['rmses'] = rmse

            all_rmse = [rmse[k1][k2] for k1 in rmse.keys() for k2 in rmse[k1].keys()]
            all_rmse = [x for x in all_rmse if np.isnan(x)==False] # exclude
            mean_rmse = np.mean(all_rmse)
            self.submissions[i]['mean_rmse'] = mean_rmse

    def summary(self, show="all"):
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])
        print("| Rank | User Id | Submitted Alias | Week | created on | status |  RMSE | zscore | mean Rank |")
        for i, rank  in enumerate(ranks):
            sub = self.submissions[rank]
            print("|%s | %10s | %5s |%5s |%s|%s|  %10.6s  | %10.6s |%10.6s |" %
                    (i+1, sub['userId'], sub['submitterAlias'],sub['week'],
                sub['substatus']['status'],  sub['createdOn'],  sub['mean_rmse'],sub['ranking'],sub['zscore']))

    def summary_final(self):
        ranks = np.argsort([sub['ranking'] for sub in self.submissions])
        print("| Rank | Team name | SynapseId Id | mean RMSE | mean Rank | mean zscore |")
        print("|----|-----|-----|----|----|------|")

        results = {}
        for i, rank  in enumerate(ranks):
            sub = self.submissions[rank]
            print("|%s | %20s | %20s | %20s | %20s | %10.6s  | %10.6s |%10.6s |" %
                    (i+1, sub['submitterAlias'], sub['userId'],
                        sub['substatus']['id'], sub['substatus']['entityId'],
                        sub['mean_rmse'],sub['ranking'],sub['zscore']))
            results[sub['submitterAlias']] = i+1
        return results



# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Simple class to handld R packages

"""
from biokit.rtools import RSession


__all__ = ['RTools']


class RTools(object):
    """A base class to handle the R session and its verbosity"""
    def __init__(self, verboseR):
        assert verboseR in [True, False]
        self._verboseR = verboseR
        self.session = RSession(verbose=self.verboseR)
       
    def _get_verboseR(self):
        return self._verboseR
    def _set_verboseR(self, value):
        assert value in [True, False]
        self._verboseR = value
        self.session.dump_stdout = value
    verboseR = property(_get_verboseR, _set_verboseR)




from dreamtools.dream8.D8C2.sc2 import D8C2_sc2
from dreamtools.dream8 import D8C2
import os

def test_sc2():

    filename = os.sep.join([D8C2.__path__[0], 'data', 'test_sc2.csv'])
    s = D8C2_sc2(filename)
    s.run()
    assert s.df['SC_m']['bestPerformer'] == 0.45085234093637499564




from dreamtools.dream8.D8C2.sc1 import D8C2_sc1
from dreamtools.dream8 import D8C2
import os





def test_sc1():
    filename = os.sep.join([D8C2.__path__[0], 'data', 'test_sc1.csv'])
    s = D8C2_sc1(filename)
    s.run()
    assert s.df.ix['yourSubmission']['meanPCI'] == 0.513854









# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################
import pickle
import glob
import os

import pylab

import numpy as np
from scipy.stats import kstest

#from scoring import HPNScoringNetwork
import easydev

__all__ = ["AUCnull"]

class AUCnull(object):
    """Loads all AUCS generated from a ramdom distribution.

    .. warning:: USED by the scoring function.


        >>> import sc1a_tools
        >>> import scoring
        >>> s = scoring.HPNScoringNetwork()
        >>> null = sc1a_tools.AUCnull(s.valid_cellLines, s.valid_ligands, verbose=False)
        >>> #filename = 'sc1a_null_aucs_mean_sigma.dat'
        >>> #filename = easydev.get_share_file("dreamtools", "data/dream8hpn", filename)
        >>> null.loadaucs(filename=None)
        >>> mean = null.get_mean_dict()
        >>> sigma = null.get_sigma_dict()


    """
    def __init__(self, valid_cellLines, valid_ligands, verbose=True):
        self.ligands = valid_ligands[:]
        self.cellLines = valid_cellLines[:]
        self.verbose = verbose

    def loadaucs(self, pickleName=None):
        if pickleName == None:
            pickleName = 'sc1a_null_aucs_mean_sigma.dat'
            pickleName = easydev.get_share_file("dreamtools", "data/dream8hpn",
                    pickleName)
        self.aucs = pickle.load(open(pickleName, "r"))

    def saveaucs(self, pickleName="sc1a_null_aucs.dat"):
        pickle.dump(self.aucs, open(pickleName, "w"))

    def loaddata(self, directory=None, tag="AUC"):
        """Loads bunch of AUCS stored in files

        files should be tagged as tag_cellline_ligand.* and contain a single
        column with all AUCs. This is convenientn if you ran many simulation
        that stores AUC in different files

        """
        self.data = {}
        for c in self.cellLines:
            self.data[c] = {}
            for l in self.ligands:
                self.data[c][l] = []

        for l in self.ligands:
            for c in self.cellLines:
                if self.verbose:print("Combining all data related to %s/%s" % (c,l))
                if directory:
                    filenames = glob.glob(directory +os.sep+"%s_%s_%s*" % (tag, c,l))
                else:
                    filenames = glob.glob("%s_%s_%s*" % (tag, c,l))

                if self.verbose: print("  found %s files\n  reading " % len(filenames))

                for filename in filenames:
                    fh = open(filename, "r")
                    data = fh.read().strip().split()
                    data = [float(x) for x in data]

                    try:
                        self.data[c][l].extend(data)
                    except:
                        self.data[c][l] = data
        self._compute_params()

    def _compute_params(self):
        self.aucs = {}
        for c in self.cellLines:
            self.aucs[c] = {}
            for l in self.ligands:
                self.aucs[c][l] = {"mean":np.mean(self.data[c][l]),
                                    "sigma":np.std(self.data[c][l])}
    def get_mean(self):
        """returns all mean AUCS in a matrix structure"""
        mean = np.array(np.zeros((4,8)))
        for i,c in enumerate(self.cellLines):
            for j,l in enumerate(self.ligands):
                mean[i][j] = self.aucs[c][l]['mean']
        return mean

    def get_sigma(self):
        """returns all std AUCS in a matrix structure"""
        sigma = np.array(np.zeros((4,8)))
        for i,c in enumerate(self.cellLines):
            for j,l in enumerate(self.ligands):
                sigma[i][j] = self.aucs[c][l]['sigma']
        return sigma

    def get_mean_dict(self):
        """returns all mean AUCS in a dictionary structure"""
        mean = {}
        for c in self.cellLines:
            mean[c] = {}
            for l in self.ligands:
                mean[c][l] = self.aucs[c][l]['mean']
        return mean

    def get_sigma_dict(self):
        """returns all std AUCS in a dictionary structure"""
        sigma = {}
        for c in self.cellLines:
            sigma[c] = {}
            for l in self.ligands:
                sigma[c][l] = self.aucs[c][l]['sigma']
        return sigma

    def get_ks(self):
        D = np.array(np.zeros((4,8)))
        pvalue = np.array(np.zeros((4,8)))
        for i,c in enumerate(self.cellLines):
            for j,l in enumerate(self.ligands):
                data = self.data[c][l]
                ks = kstest(data, 'norm', args=([np.mean(data), np.std(data)]))
                D[i][j] = ks[0]
                pvalue[i][j] = ks[1]
        return D, pvalue

    def pcolor(self, mode="mean", vmin=None, vmax=None):
        """

        If you loaded the pickle data sets with only mean and sigma, the D and
        pvalue mode cannot be used.

        """
        from pylab import clf, xticks, yticks, pcolor, colorbar, flipud, log10
        if mode == "mean":
            data = self.get_mean()
        elif mode == "sigma":
            data = self.get_sigma()
        elif mode == "D":
            data = self.get_ks()[0]
        elif mode == "pvalue":
            data = self.get_ks()[1]
        clf();

        if mode == "pvalue":
            pcolor(log10(flipud(data)), vmin=vmin, vmax=vmax);
        else:
            pcolor(flipud(data), vmin=vmin,vmax=vmax);

        colorbar()
        xticks([x+0.5 for x in range(0,8)], self.ligands, rotation=90)
        cellLines = self.cellLines[:]
        cellLines.reverse()
        yticks([x+0.5 for x in range(0,4)], cellLines, rotation=0)

    def plot_auc_histograms(self, directory):
        #from scoring import HPNScoringNetwork
        #s = HPNScoringNetwork()

        self.loaddata(directory=directory, tag="AUC")

        pylab.figure(1)
        pylab.clf()
        for i,c in enumerate(self.cellLines):
            for j,l in enumerate(self.ligands):
                pylab.subplot(4,8,i*8+j+1)
                pylab.hist([x for x in self.data[c][l] if np.isnan(x)==False], 50, normed=True)
                pylab.title("%s/%s" % (c,l))
                pylab.xlim([0.,1])
                pylab.grid(True)

    def plot_aupr_histograms(self, directory):
        #from scoring import HPNScoringNetwork
        #s = HPNScoringNetwork()
        self.loaddata(directory=directory, tag="AUPR")
        pylab.figure(1)
        pylab.clf()
        for i,c in enumerate(self.cellLines):
            for j,l in enumerate(self.ligands):
                pylab.subplot(4,8,i*8+j+1)
                pylab.hist([x for x in self.data[c][l] if np.isnan(x)==False], 50, normed=True)
                pylab.title("%s/%s" % (c,l))
                pylab.xlim([0.,1])
                pylab.grid(True)


# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""D5C2 challenge scoring functions

:Credits: Based on TF_web.pl (perl version) provided by Raquel Norel (Columbia University/IBM)
   that is used wihtin the web server http://www.ebi.ac.uk/saezrodriguez-srv/d5c2/cgi-bin/TF_web.pl
"""
import os
from os.path import join as pj
import zipfile
import StringIO
import collections
import numpy as np
import pandas as pd
import tempfile

from easydev import progress_bar
from dreamtools.core.ziptools import ZIP
from dreamtools.core.challenge import Challenge
from dreamtools.core.rocs import ROCDiscovery
from dreamtools.core.downloader import Downloader



class D5C2(Challenge):
    """A class dedicated to D5C2 challenge


    ::

        from dreamtools import D5C2
        s = D5C2()

        # You can get a template from www.synapse.org page (you need to register)
        s.download_templates() 
        s.score('templates.txt.gz') # takes about 5 minutes
        s.get_table()
        s.plot()

    Data and templates are downloaded from Synapse. You must have a login.

    """
    def __init__(self, tmpdir=None, Ntf=66):
        """.. rubric:: constructor
        
        :param Ntf: not to be used. Used for fast testing and debugging
        :param tmpdir: a local temporary file if provided. 
        """
        super(D5C2, self).__init__('D5C2')
        self._path2data = os.path.split(os.path.abspath(__file__))[0]
        self.Ntf = Ntf
        self.tmpdir = tmpdir               # directory where to save the results

        self._dvs = {}
        self._dvps = {}
        self._probes = {}

    def score(self, prediction_file):
        """Compute all results and compare user prediction with all official participants
        
        This scoring function can take a long time (about 5-10 minutes). 
        """

        self.init() # this provides a temporary file
        self.prediction_file = prediction_file
        print('Loading user data set (step 1 out of 5)')
        self._loading_user_data()

        print('Loading the gold standard and user prediction. Takes a few seconds (step 2 out 5')
        self.download_all_data()

        print('\nProcessing\nSplitting data sets (step 3 out of 5)')
        self._split_data()
        print('\nComputing probes (step 4 out of 5)')
        self._preprocessing()
        print('\nComputing performances (step 5 out of 5)')
        self._processing()

    def init(self):
        """Creates the temporary directory and the sub directories.

        Behaviour differs whether the directory was provided
        in the constructor or not.
        """
        if self.tmpdir is None:
            self.tmpdir = tempfile.mkdtemp()
        else:
            try:
                os.mkdir(self.tmpdir)
            except:
                pass

        for directory in ['Data', 'Out']:
            this = pj(self.tmpdir, directory)
            if os.path.exists(this) is False:
                os.mkdir(this)

    def download_all_data(self):
        """Download all large data sets from Synapse"""
        pb = progress_bar(5)
        # load the large gold standard file from D5C2 synapse main page
        filename = self._download_data('DREAM5_GoldStandard_probes.zip', 'syn2898469')
        pb.animate(1)
        z = ZIP()
        z.loadZIPFile(filename)
        data = z.read('Answers.txt')
        self.gs = pd.read_csv(StringIO.StringIO(data), sep='\t')

        # download 4 other filenames from dreamtools synapse project
        self._download_data('all_8mers.txt', 'syn4483185')
        pb.animate(2)
        self._download_data('8mers_gs.txt', 'syn4483187')
        pb.animate(3)
        self._download_data('probe35_gs.txt', 'syn4483184')
        pb.animate(4)
        self._download_data('probes35.txt', 'syn4483183')
        pb.animate(5)
    
    def download_templates(self):
        """Download a template from synapse into ~/config/dreamtools/dream5/D5C2

        :return: filename and its full path
        """
        filename = self._download_data('templates.txt.gz', 'syn4483192')
        return filename

    def _download_data(self, name, synid):
        filename = self.directory + os.sep + name
        if os.path.exists(filename) is False:
            # must download the data now
            print("File %s not found. Downloading from Synapse. You must have a login." % filename)
            d = Downloader(self.nickname)
            d.download(synid)

        return filename

    def _split_data(self, precision=6):
        """precision is to get same results as in the original perl script"""
        mask = self.gs.Flag == 0
        self.user_data_clean = self.user_data[mask].copy()
        print('Splitting the user data set and removing flagged data (%s out of %s)' % (self.gs.shape[0] - mask.sum(), self.gs.shape[0]))
        self.gs_clean = self.gs[mask].copy()
        # local aliases
        gs = self.gs_clean
        user_data = self.user_data_clean

        pb = progress_bar(self.Ntf, interval=1)
        for tf_index in range(1, self.Ntf + 1):
            this_tf = 'TF_%s'  % tf_index
            tf_gs = gs.query("Id == @this_tf").Answer
            tf_user = user_data.query("TF_Id == @this_tf").Signal_Mean
            df = pd.concat([tf_gs, tf_user], axis=1)
            df.to_csv(self._setfile(tf_index, 'Data'), index=False, sep='\t',
                      header=False, float_format="%f")
            pb.animate(tf_index)

    def _loading_user_data(self):
        """Get the file from the form, save it, decompress it."""

        # could be either gz or zip
        import mimetypes
        itemtype = mimetypes.guess_type(self.prediction_file)[1]
        
        if itemtype == 'gzip':
            import gzip
            fh = gzip.open(self.prediction_file, 'rb')
            data = fh.read()
            fh.close()
        elif itemtype == 'zip':
            z = zipfile.ZipFile(self.prediction_file)
            assert len(z.filelist) == 1, "zipped archive should contain only 1 file"
            # extract in byte
            data = z.read(z.filelist[0])
            self.prediction_file_unzipped = 'tmp_' + z.filelist[0].filename
        else:
            raise IOError("input file must be gzipped or zipped")

        df = pd.read_csv(StringIO.StringIO(data), sep='\t');# engine='python')
        self.user_data = df

    def _preprocessing(self):
        """Create temporary files for before further processing

        :return: nothing
        """
        # Read file octomers gold standard
        filename = self.directory + os.sep + '8mers_gs.txt'
        self.octomers_gs = pd.read_csv(filename, sep='\t', header=None)

        # Read file octomers 
        filename = self.directory + os.sep + 'all_8mers.txt'
        self.octomers = pd.read_csv(filename, sep='\t', header=None)  # contains reverse complemtn
        self.octomers.columns = ['octomer','octomerRC']

        # Read probes gs
        filename = self.directory + os.sep + 'probe35_gs.txt'
        self.probes_gs = pd.read_csv(filename, header=None, sep='\t')
        self.probes_gs.columns = ['Id', 'Sequence']

        # reads probes (sequences)
        print('Reading probes')
        filename = self.directory + os.sep + 'probes35.txt'
        # just one column so no need for a separator
        probes = pd.read_csv(filename)

        # Extract information (first and third column of pred.txt)
        df = self.user_data[['TF_Id', 'Signal_Mean']].copy()
        df['Signal_Mean'] = df['Signal_Mean'].map(lambda x: round(x,6))

        # data.txt is paste of probes35.txt and val.txt
        data = pd.concat([probes, df], axis=1)

        # Creates probes/TF_1.dat that contains the sequence from the GS and the answer from the user
        # for each TF
        print('Creating probes/TF_1.csv + sorting')
        pb = progress_bar(self.Ntf, interval=1)
        for i in range(1, self.Ntf+1):
            # could use a groupby here ? faster  maybe
            tag = 'TF_%s' % i
            sequence = data[['Sequence']].ix[self.gs.Id==tag]
            answer = data.Signal_Mean[data.TF_Id == tag]
            df = pd.concat([sequence, answer], axis=1)
            df.sort(columns=['Signal_Mean', 'Sequence'], ascending=[False, False], inplace=True)
            df['Signal_Mean'] = df['Signal_Mean'].map(lambda x: round(x,6))

            self._probes[i] = df
            pb.animate(i)

    def _setfile(self, index, directory):
        return self.tmpdir + os.sep + directory + os.sep + 'TF_%s' % index + '.csv'

    def _processing(self):
        """

        :return:
        """
        ########################################  1 Create the Out/TF_XX.dat files
        octomers = self.octomers.octomer
        octomersRC = self.octomers.octomerRC
        mapping1  = dict([(k,v) for k,v in zip(octomers.values, octomersRC.values)])
        mapping2  = dict([(k,v) for k,v in zip(octomersRC.values, octomers.values)])
        keys = tuple(sorted(octomers.values))

        lm = set(octomers.values)

        pb = progress_bar(self.Ntf, interval=1)
        pb.animate(0)
        for tf_index in range(1, self.Ntf + 1):
            tf = self._probes[tf_index]
            tf.columns = ['Sequence', 'Score']
            ids = collections.defaultdict(list)
            ###### TODO: most of the time is spent in the "for curR in generator" loop
            for seq, score in zip(tf.Sequence, tf.Score):
                # scan the sequence by chunk of octomers using a generator
                # for speed (although gain is small)
                generator = (seq[i:i+8] for i in xrange(0,28))
                for curR in generator:
                    if mapping1.has_key(curR) is False:
                        curR = mapping2[curR]
                    ids[curR].append(score)
                # Using a set does not help speeding up the code
                #for curR in generator:
                #    if curR not in lm:
                #        curR = mapping2[curR]
                #    ids[curR].append(score)

            # now let us build the new dataframe for the indices found
            df = pd.DataFrame({0:[k for k in  ids.keys()],
                               1:[np.median(v) for v in ids.values()]})
            df.sort(columns=[1,0], ascending=[False, False], inplace=True)
            df[1] = df[1].map(lambda x: round(x,6))

            df.to_csv(self._setfile(tf_index, 'Out'), sep=' ', index=False, header=None, float_format="%.6f")
            pb.animate(tf_index)
        print("")
        ################################################# 2 create the DVP

        pb = progress_bar(self.Ntf, interval=1)
        for tf_index in range(1,self.Ntf+1):
            tag = 'TF_%s' % tf_index
            tf_probes = list(self.probes_gs.ix[self.probes_gs.groupby('Id').groups[tag]].Sequence)

            tf = self._probes[tf_index]
            dv = tf.Sequence.apply(lambda x: x in tf_probes).astype(int)
            self._dvps[tf_index] = dv
            pb.animate(tf_index)
        print("")

        ########################################################## DV
        gs_octomers = self.octomers_gs.copy()
        gs_octomers.columns = ['id', 'octomer']
        pb = progress_bar(self.Ntf, interval=1)
        for tf_index in range(1,self.Ntf+1):
            tag = 'TF_%s' % tf_index
            tf_octomers = list(gs_octomers.ix[gs_octomers.groupby('id').groups[tag]].octomer)
            tf = pd.read_csv(self._setfile(tf_index, "Out"), sep=" ",
                             header=None)
            tf.columns = ['Octomer', 'Score']
            dv = tf.Octomer.apply(lambda x: x in tf_octomers).astype(int)

            # Stores the dataframe
            self._dvs[tf_index] = dv
            pb.animate(tf_index)

    def compute_statistics(self):
        """Returns final results of the user predcition
        
        :return: a dataframe with various metrics for each transcription factor.
        
        Must call :meth:`score` before.

        """
        data = {'Pearson': [],
                'Spearman': [],
                'Pearson_Log': [],
                "AUROC_8mer": [],
                "AUPR_8mer": [],
                "AUROC_probe": [],
                "AUPR_probe": []}

        pb = progress_bar(self.Ntf, interval=1)
        for tf_index in range(1, self.Ntf + 1):
            dfdata = pd.read_csv(self._setfile(tf_index, "Data"), sep='\t', header=None)
            pearson = dfdata.corr('pearson').ix[0,1]
            spearman = dfdata.corr('spearman').ix[0,1]
            pearsonLog = np.log10(dfdata).corr('pearson').ix[0,1]

            data['Pearson'].append(pearson)
            data['Pearson_Log'].append(pearsonLog)
            data['Spearman'].append(spearman)


            dvdata = self._dvs[tf_index]

            #dvdata = pd.read_csv(self._setfile(tf_index, "DV"), index_col=False, header=None)
            r = ROCDiscovery(dvdata.values)
            rocdata = r.get_statistics()
            auroc = r.compute_auc(roc=rocdata)
            aupr = r.compute_aupr(roc=rocdata)
            data['AUROC_8mer'].append(auroc)
            data['AUPR_8mer'].append(aupr)

            dvdata = self._dvps[tf_index] 
            #dvdata = pd.read_csv(self._setfile(tf_index, 'DVP'), index_col=False, header=None)
            r = ROCDiscovery(dvdata.values)
            rocdata = r.get_statistics()
            auroc = r.compute_auc(roc=rocdata)
            aupr = r.compute_aupr(roc=rocdata)
            data['AUROC_probe'].append(auroc)
            data['AUPR_probe'].append(aupr)
            pb.animate(tf_index)

        df =  pd.DataFrame(data)
        df = df[['Pearson', u'Spearman', u'Pearson_Log', u'AUROC_8mer', u'AUPR_8mer', u'AUROC_probe', u'AUPR_probe']]

        return df

    def get_table(self):
        """Return table with user results from the user and participants

        There are 14 participants as in the Leaderboard found here 
        https://www.synapse.org/#!Synapse:syn2887863/wiki/72188


        :return: a dataframe with different metrics showing performance of the submission
            with respect to other participants.


        ::

            table = s.get_table()
            with open('test.html', 'w') as fh:
                fh.write(table.to_html(index=False))

        """
        userdf = self.compute_statistics()
        userdf = userdf.mean().to_frame().T
        userdf.index = [20] # there are 20 participants, let us add this user as the 21st
        userdf = userdf[['Pearson', 'Pearson_Log', 'Spearman', 'AUROC_8mer', 'AUPR_8mer']]
        userdf['Team'] = 'Your team'
        userdf['Model type'] = 'Your model'

        filename = os.sep.join([self._path2data, "data", "d5c2_data_table.csv"])
        participants = pd.read_csv(filename, sep='\t', index_col=0)
        table = pd.concat([participants, userdf])

        # compute ranks based on those columns. Using method first to be in agreement with the server
        rank_columns = ['Pearson', 'Pearson_Log', 'Spearman', 'AUPR_8mer', 'AUROC_8mer']
        ranks = table[rank_columns]
        mean_ranks = ranks.rank(ascending=False, method='first').mean(axis=1)
        table['Final Rank (average)'] = mean_ranks.values
        table['Final Rank'] = table['Final Rank (average)'].rank(method='first')
        table = table[['Team', 'Model type', 'Final Rank', 'Final Rank (average)'] + rank_columns]
        table = table.sort(columns=['Final Rank'])
        return table

    def _get_table(self):
        userdf = self.compute_statistics()
        userdf = userdf.mean().to_frame().T
        userdf.index = [20] # there are 20 participants, let us add this user as the 21st

        # load all data from participants for comparison
        filename = os.sep.join([self._path2data, "data", "d5c2_data_plot.csv"])
        participants = pd.read_csv(filename, sep='\t', )

        df = pd.concat([participants, userdf])
        return (df, userdf)

    def plot(self):
        """Show the user prediction compare to 20 other participants"""
        df, userdf = self._get_table()

        import pylab
        pylab.clf();
        pylab.subplot(2,2,1)
        pylab.plot(df.AUROC_8mer, df.AUPR_8mer, marker='+', color='k', lw=0, markersize=10)
        pylab.plot(userdf.AUROC_8mer, userdf.AUPR_8mer, marker='s', color='b', lw=0, markersize=10)
        pylab.xlim([0,1])
        pylab.ylim([0,1])
        pylab.xlabel('AUROC octamers')
        pylab.ylabel('AURPR octamers')

        pylab.subplot(2,2,2)
        pylab.plot(df.AUROC_probe, df.AUPR_probe, marker='+', color='k', lw=0, markersize=10)
        pylab.plot(userdf.AUROC_probe, userdf.AUPR_probe, marker='s', color='b', lw=0, markersize=10)
        pylab.xlim([0,1])
        pylab.ylim([0,1])
        pylab.xlabel('AUROC probes')
        pylab.ylabel('AURPR probes')

        pylab.subplot(2,2,3)
        pylab.plot(df.Pearson, df.Spearman, marker='+', color='k', lw=0, markersize=10)
        pylab.plot(userdf.Pearson, userdf.Spearman, marker='s', color='b', lw=0, markersize=10)
        pylab.xlim([0,1])
        pylab.ylim([0,1])
        pylab.xlabel('Pearson probes')
        pylab.ylabel('Spearman probes')

        pylab.subplot(2,2,4)
        pylab.plot(df.Pearson, df.Pearson_Log, marker='+', color='k', lw=0, markersize=10)
        pylab.plot(userdf.Pearson, userdf.Pearson_Log, marker='s', color='b', lw=0, markersize=10)
        pylab.xlim([0,1])
        pylab.ylim([0,1])
        pylab.xlabel('Pearson probes')
        pylab.ylabel('Log Pearson probes')

    def cleanup(self):
        """Remove the temporary directory"""
        import shutil
        shutil.rmtree(self.tmpdir)



from .core.sageutils import Login
from .core import settings
import os
configuration = settings.DreamToolsConfig()
dreampath = configuration.user_config_dir


def create_dreamtools_config_directories(maindir, subdirectories):
    import os
    cfg = configuration.user_config_dir
    path = cfg + os.sep + maindir
    if os.path.isdir(path) is False:
        print("Creating directories in %s for %s" % (cfg, maindir) )
        os.mkdir(path)
    for directory in subdirectories:
        subdir = path + os.sep +directory
        if os.path.isdir(subdir) is False:
            print(" - creating %s" % subdir)
            os.mkdir(subdir)

from dream7.D7C1.scoring import D7C1
from dream5.D5C2.scoring import D5C2
from dream8.D8C2.sc1 import D8C2_sc1
from dream8.D8C2.sc2 import D8C2_sc2


from dreamtools.dream8.D8C1 import scoring
from dreamtools.dream8.D8C1 import d8c1path
import os

def test_scoring_sc1a():
    filename = os.sep.join([d8c1path, 'submissions', 'sc1a',
        'limax-Network.zip'])
    s = scoring.HPNScoringNetwork(filename)

    s.compute_all_aucs()
    assert s.get_auc_final_scoring() == 0.77107383364675341

    filename = os.sep.join([d8c1path, 'submissions', 'sc1a',
        'DC_GFP-Network.zip'])
    s.load_submission(filename)
    s.compute_all_aucs()
    assert s.get_auc_final_scoring() == 0.78202740472227761

    # just try some other functions
    s.get_null_distribution(2)
    s.plot_all_rocs()


def test_scoring_sc1b():
    filename = os.sep.join([d8c1path, 'submissions', 'sc1b',
        'NMSUSongLab-Network-Insilico.zip'])
    s = scoring.HPNScoringNetworkInsilico(filename)
    s.compute_score()
    assert s.auc == 0.76422010245539662


def test_scoring_sc2a():
    filename = os.sep.join([d8c1path, 'submissions', 'sc2a',
        'guanlab10-Prediction.zip'])
    s = scoring.HPNScoringPrediction(filename)
    s.compute_all_rmse()
    assert s.get_mean_rmse() == 0.45297349456396707


def test_scoring_sc2b():
    filename = os.sep.join([d8c1path, 'submissions', 'sc2b',
        'CGR-Prediction-Insilico.zip'])
    s = scoring.HPNScoringPredictionInsilico(filename)
    s.compute_all_rmse()
    #assert s.get_mean_rmse() == 0.24554824475398279 former wrong true prediction
    assert s.get_mean_rmse() == 0.27750290989139931 





from dreamtools.dream8.D8C1 import submissions



def test_submissions_sc1a():
    s = submissions.SC1ASubmissions()
    s.load_submissions()
    assert len(s.submissions) == 74
    s.summary_final()


def test_submissions_sc1b():
    s = submissions.SC1BSubmissions()
    s.load_submissions()
    assert len(s.submissions) == 65
    s.summary_final()


def test_submissions_sc2a():
    s = submissions.SC2ASubmissions()
    s.load_submissions()
    assert len(s.submissions) == 14
    s.summary_final()


def test_submissions_sc2b():
    s = submissions.SC2BSubmissions()
    s.load_submissions()
    assert len(s.submissions) == 11
    s.summary_final()




# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Utility to download a synapse project in the dreamtools directory"""

# import os

from dreamtools.core.sageutils import Login
from dreamtools.core.challenge import Challenge


__all__ = ['Downloader']


class Downloader(Challenge, Login):
    """Factory to download gold standard files

    Download a synpase file once for all in the dreamtools directory.



    """
    def __init__(self, challenge, client=None):
        """.. rubric:: constructor

        :param str challenge: nickname of a challenge (e.g., D5C1)

        To automatically connect to synapse, create a file called .synapseConfig with this
        content::

            [authentication]
            username: email
            password: password

        """
        Challenge.__init__(self, challenge_name=challenge)
        Login.__init__(self, client=client)

    def download(self, synid):
        """Download a file into the dreamtools directory
        
        :param synid: a valid synapse id (e.g., syn123456)

        You must have a login on synapse website. 
        """
        assert synid.startswith('syn'), "synid must be a valid synapse identifier e.g., syn123456"
        self.client.get(synid, downloadLocation=self.directory)





from dreamtools.dream8.D8C1 import downloads



def test_downloads_submissions():
    d = downloads.SubmissionsDownloader()
    d.download_all() # takes some time



def test_downloads_gs():
    d = downloads.GSDownloader()
    d.download_experimental()





# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
import zipfile


class ZIP(object):
    """Simple utility to load a ZIP file"""

    def __init__(self):
        pass

    def loadZIPFile(self, filename):
        """Loads a ZIP file

        This method uses the zipfile module and stores the data into 
        :attr:`zip_data`. The filenames contained within this archive
        can be found in :attr:`zip_filenames`. To read the data contained 
        in the first filename, type:: 

            self.zip_data.open(self.filenames[0].read()

        :param str filename: the ZIP filename to load

        """
        # USED in D8C1 and D5C2 do not change
        if zipfile.is_zipfile(filename) == False:
            raise ValueError("Input file (%s) is not a valid ZIP file. " % filename)
        else:
            self.zip_data = zipfile.ZipFile(filename)
            self.zip_filenames = self.zip_data.namelist()
            self.filenames = self.zip_filenames # keep zip_filenames for back compat


    def read(self, filename):
        return self.zip_data.read(filename)



# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""Main standalone application dreamtools-scoring"""
import os
import argparse
import sys
from easydev.console import red, purple, darkgreen
registered = {'d8c1': ['sc1a', 'sc1b', 'sc2a', 'sc2b'],
        'd8c2': ['sc1', 'sc2'],
        'd7c1':['parameter', 'topology', 'timecourse'],
        'd5c2': []}


# Define the simple scoring functions here below

def d7c1_model1_parameter(filename):
    """wrapper to score D7C1 submission (parameter model1)"""
    from dreamtools import D7C1
    s = D7C1()
    score = s.score_model1_parameters(filename)
    return {'score': score}


def d7c1_model1_prediction(filename):
    """wrapper to score D7C1 submission (prediction model1)"""
    from dreamtools import D7C1
    s = D7C1()
    score = s.score_model1_timecourse(filename)
    return {'score': score}


def d7c1_model2_topology(filename):
    """wrapper to score D7C1 submission (topology)"""
    from dreamtools import D7C1
    s = D7C1()
    score = s.score_topology(filename)
    return {'score': score}

def d5c2(filename):
    """wrapper to score D5C2 submission"""
    from dreamtools import D5C2
    s = D5C2(Ntf=2)
    s.score(filename)

    df = s.get_table()
    results = {}
    results['table']  = df
    results['user model'] = df.ix[20] # 20 is the user id
    return results


def d8c1_sc1a(filename, verbose=False):
    """wrapper to score D8C1 submission(SC1A sub challenge)"""
    from dreamtools.dream8.D8C1 import scoring, ranking
    sc1a = scoring.HPNScoringNetwork(filename,  verbose=verbose)
    sc1a.compute_all_aucs()
    rank = ranking.SC1A_ranking()
    rank.append_submission(filename)

    return {'AUROC': sc1a.get_auc_final_scoring(),
            'Rank LB': rank.get_rank_your_submission()}


def d8c1_sc1b(filename, verbose=False):
    """wrapper to score D8C1 submission(SC1B sub challenge)"""
    from dreamtools.dream8.D8C1 import scoring, ranking
    sc1b = scoring.HPNScoringNetworkInsilico(filename,  verbose=verbose)
    sc1b.compute_score()
    rank = ranking.SC1B_ranking()
    rank.append_submission(filename)
    return {'AUROC': sc1b.auc,
            'Rank LB': rank.get_rank_your_submission()}


def d8c1_sc2a(filename, verbose=False):
    """wrapper to score D8C1 submission(SC2A sub challenge)"""
    from dreamtools.dream8.D8C1 import scoring, ranking
    sc2a = scoring.HPNScoringPrediction(filename, verbose=verbose)
    sc2a.compute_all_rmse()
    rank = ranking.SC2A_ranking()
    rank.append_submission(filename)
    return {'RMSE': sc2a.get_mean_rmse(),
            'Rank LB': rank.get_rank_your_submission()}

def d8c1_sc2b(filename, verbose=False):
    """wrapper to score D8C1 submission(SC2B sub challenge)"""
    from dreamtools.dream8.D8C1 import scoring, ranking
    sc2b = scoring.HPNScoringPredictionInsilico(filename, verbose=verbose)
    sc2b.compute_all_rmse()
    rank = ranking.SC2B_ranking()
    rank.append_submission(filename)
    return {'RMSE': sc2b.get_mean_rmse(),
            'Rank LB': rank.get_rank_your_submission()}


# DREAM8 Challenge 2
def d8c2_sc1(filename, verbose=False, verboseR=False):
    """wrapper to score D8C2 submission(SC1 sub challenge)"""
    from dreamtools.dream8.D8C2 import sc1
    s = sc1.D8C2_sc1(filename, verboseR=verboseR)
    s.run()
    return {'results': s.df}


def d8c2_sc2(filename, verbose=False, verboseR=False):
    """wrapper to score D8C2 submission(SC2 sub challenge)"""
    from dreamtools.dream8.D8C2 import sc2
    s = sc2.D8C2_sc2(filename, verboseR=verboseR)
    s.run()
    return {'results': s.df}



# -------------------------------------------------- The User Interface
def print_color(txt, func_color, underline=False):
    import easydev
    try:
        if underline:
            print(easydev.underline(func_color(txt)))
        else:
            print(func_color(txt))
    except:
        print(txt)


def scoring(args=None):
    """This function is used by the standalone application called dreamscoring

    ::

        dreamscoring-scoring --help

    """
    import easydev
    d = easydev.DevTools()

    if args == None:
        args = sys.argv[:]
    user_options = Options(prog="dreamtools-scoring")

    if len(args) == 1:
        user_options.parse_args(["prog", "--help"])
    else:
        options = user_options.parse_args(args[1:])

    if options.challenge is None:
        if len(registered[options.challenge])!=0 and options.sub_challenge is None:
            print_color('--challenge and --sub-challenge must be provided', red)
            sys.exit()

    try:
        d.check_param_in_list(options.challenge, registered.keys())
    except ValueError as err:
        txt = "DreamScoring error: unknown challenge name (%s) or not yet implemented\n" % options.challenge
        txt += "--->" + err.message
        print_color(txt, red)
        sys.exit()

    if options.sub_challenge is not None:
        try: 
            d.check_param_in_list(options.sub_challenge, registered[options.challenge])
        except ValueError as err:
            txt = "DreamScoring error: unknown sub challenge or not yet implemented"
            txt += "--->" + err.message
            print_color(txt, red)
            sys.exit()

    if options.filename is None:
        txt = "---> filename not provided. You must provide a filename with correct format\n"
        txt += "Format are explained on DreamTools website or Synapse website\n"
        txt += "https://github.com/dreamtools/dreamtools, or http://www.synapse.org\n"
        print_color(txt, red)
        sys.exit()

    if os.path.exists(options.filename) is False:
        raise IOError("file %s does not seem to exists" % options.filename)

    print_color("Dreamtools scoring", purple, underline=True)
    print('Challenge %s (sub challenge %s)\n\n' % (options.challenge, options.sub_challenge))

    res = '??'

    if options.challenge not in registered.keys():
        raise ValueError('Invalid challenge name. Choose one of %s' % registered.keys())

    if options.challenge == 'd8c1':
        if options.sub_challenge == 'sc1a':
            res = d8c1_sc1a(options.filename, verbose=options.verbose)
        elif options.sub_challenge == 'sc1b':
            res = d8c1_sc1b(options.filename, verbose=options.verbose)
        elif options.sub_challenge == 'sc2a':
            res = d8c1_sc2a(options.filename, verbose=options.verbose)
        elif options.sub_challenge == 'sc2b':
            res = d8c1_sc2b(options.filename, verbose=options.verbose)
    elif options.challenge == 'd8c2':
        if options.sub_challenge == 'sc1':
            res = d8c2_sc1(options.filename, verbose=options.verbose)
        if options.sub_challenge == 'sc2':
            res = d8c2_sc2(options.filename, verbose=options.verbose)
    elif options.challenge == 'd7c1':
        if options.sub_challenge == 'parameter':
            res = d7c1_model1_parameter(options.filename)
        if options.sub_challenge == 'prediction':
            res = d7c1_model1_prediction(options.filename)
        if options.sub_challenge == 'topology':
            res = d7c1_model2_topology(options.filename)
    elif options.challenge == 'd5c2':
        res = d5c2(options.filename)

    txt = "Solution for %s in challenge %s" % (options.filename, options.challenge)
    if options.sub_challenge is not None:
        txt += " (sub-challenge %s)" % options.sub_challenge
    txt += " is :\n"

    for k in sorted(res.keys()):
        txt += darkgreen("     %s: %s\n" %(k, res[k]))

    print(txt)


class Options(argparse.ArgumentParser):
    description = "tests"
    def __init__(self, version="1.0", prog=None):

        usage = """usage: python %s --challenge d8c1 --sub-challenge sc1a --submission <filename>""" % prog
        usage += """      python %s --challenge d5c2 --submission <filename>""" % prog
        epilog="""Author(s):

        - Thomas Cokelaer: DreamTools package and framework including tests and docs
        - Thomas Cokelaer: D8C1, Parameter Estimation D6 and D7
        - Federica Eduati: original R scripts for D8C2
        - Pablo Meyer: Parameter Estimation D6 and D7


Source code on: https://github.com/dreamtools/dreamtools
Issues or bug report ? Please fill an issue on http://github.com/dreamtools/dreamtools/issues """
        description = """General Description:
    You must provide the challenge nickname (e.g., d8c1 for Dream8, Challenge 1) and
    if there were several sub-challenges, you also must provide the sub-challenge
    nickname (e.g., sc1). Finally, the submission has to be provided. The format must
    be in agreement with the description of the challenge itself.

    Help and documentation about the templates may be found either within the online
    documentation http://pythonhosted.org/dreamtools/ or within the source code
    hosted on github http://github.org/dreamtools/dreamtools

    Registered challenge so far (and sub-challenges) are:
    """
        description +="\n"
        for c in registered.keys():
            description +=  "    - " + c + ": "
            for s in registered[c]:
                description += s + " "
            description += "\n"

        super(Options, self).__init__(usage=usage, version=version, prog=prog, 
                epilog=epilog, description=description,
                formatter_class=argparse.RawDescriptionHelpFormatter)
        self.add_input_options()

    def add_input_options(self):
        """The input oiptions.

        Default is None. Keep it that way because otherwise, the contents of
        the ini file is overwritten in :class:`apps.Apps`.
        """
        group = self.add_argument_group("General", 'General options (compulsary or not)')

        group.add_argument("--challenge", dest='challenge',
                         default=None, type=str, 
                         help="nickname of the challenge (e.g., d8c1 stands for"
                         "dream8 challenge 1). Challenge nicknames can be found on"
                         "dreamchallenges.org.")
        group.add_argument("--sub-challenge", dest='sub_challenge', 
                         default=None, type=str,
                         help="Name of the data files")
        group.add_argument("--verbose", dest='verbose',
                         action="store_true",
                         help="verbose option.")
        group.add_argument("--submission", dest='filename',
                         help="submission/filename to score.")
        group.add_argument("--filename", dest='filename',
                         help="submission/filename to score.")
        #group.add_argument("--help", dest='help',
        #                 action="store_true",
        #                 help="this help.")

if __name__ == "__main__":
    scoring(sys.argv)



# -*- python -*-
#
#  This file is part of DreamTools software
#
#  Copyright (c) 2014-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.org/dreamtools
#
##############################################################################
"""A module dedicated to synapse

The class :class:`SynapseClient` is a specialised class built upon synapseclient
package, which source code is on GIT::
 
    git clone git://github.com/Sage-Bionetworks/synapsePythonClient.git
    cd synapsePythonClient
    python setup.py install

When I started to build dream8 synapse pages and leaderboards, the synapseclient
was still in development. In order to test the system, I built this class
(SynapseClient) to add high level functionalities. These functionalities have
been included later in the synapseclient. After successive cleanups, I've still
have a couple of methods that may be redundant with synapse or not...

This class may be removed but for now it is used

::

    >>> from dreamtools.core import sageutils
    >>> s = sageutils.SynapseClient()


"""
import synapseclient
from synapseclient import entity, Submission, SubmissionStatus


__all__ = ["SynapseClient", "Login"]


class Login(object):
    """A simple class to login to synapse

    :param client: Connection to synapse takes a
        couple of seconds. This may be too much if in a debugging mode or
        accessing to synapse from different places. The login can be instantiate
        with an existing instance of SynapseClient, if which case, the instance
        creation is fast. Otherwise, the default behaviour is to create a new
        connection.

    :: 

        >>> from dreamtools.sageutils import Login
        >>> l = Login()
        This is a SynapseClient built on top of Synapse class. 
        Trying to login automatically. 
        Welcome, *****************
        You're logged in Synapse
        Welcome, cokelaer@ebi.ac.uk!

        In [10]: l = sageutils.Login(l)

    """
    def __init__(self, client=None, username=None, password=None):
        if client is None:
            self.client = SynapseClient(username, password)

        else:
            self.client = client

# note that synapseclient.Synapse does not inherit from object, hence the
# additional base class here below that contains Synapse and object as base
# classes
class SynapseClient(synapseclient.Synapse, object):
    """This class inherits all methods from synapseClient. 

    Be aware that most of the functionalities are now available in synapseclient
    itself. So, most of the methods that were written are hidden (double
    underscore) and may be removed in the future. 

    The only remaining feature is the automatic login, and simple version of the
    downloadSubmission method. There is also a :meth:`json` method used
    throughout the dream8hpn code.

    """
    def __init__(self, username=None, password=None):
        """.. rubric:: Constructor

        :param username: your synapse usename
        :param password: your synapse password

        You can create create a file called **.synapseConfig** (note the dot)
        in your home directory and add something like::

            [authentication]
            username: yourlogin
            password: yourpassword


        """
        super(SynapseClient, self).__init__()
        # TODO: should be moved to Login ??
        print("This is a SynapseClient built on top of Synapse class. ")
        print("Trying to login automatically. ")
        try:
            self.login(username, password)
            print("You're logged in Synapse")
        except Exception, e:
            print(e)
            print("Failed to login automatically. Please try manual login")
            raise Exception

    def __getDataPath(self, entity, version=1):
        """

        obsolet .to be deleted. Feb 2014

        """
        try:
            eid = entity['id']
            e = self.get(eid, version=version, downloadFile=True)
        except:
            e = self.get(entity, version=version, downloadFile=True)
        return e['path']

    def downloadSubmissionAndFilename(self, sub, downloadFile=True, **kargs):
        """Return filename of a submission downloaded from synapse. 

        :param sub:  A submission (as a dictionary).
        :param version:  The specific version to get. Defaults to the most recent version.
        :param downloadFile: Whether associated files(s) should be downloaded.  
             Defaults to True. If set to False, downloadLocation and ifcollision are ignored
        :param downloadLocation: Directory where to download the Synapse File Entity.
            Defaults to the local cache.
        :param ifcollision: Determines how to handle file collisions.
            May be "overwrite.local", "keep.local", or "keep.both".
            Defaults to "keep.both".

        .. warning:: ifcollision does not seem to work (0.5.1)

        """

        if isinstance(sub, dict) == False:
            raise TypeError("input must be a submission (dictionary)")
            

        if downloadFile == False:
            filename = self.getSubmission(sub, downloadFile=False)['filePath']
        else:
            filename = self.getSubmission(sub, downloadFile=True, **kargs)['filePath']

        return filename

    def __setProvenance(self, entity, name):
        """Set provenance

        :param entity: a valid entity object
        :param name: name to set on an activity

        ::

            s.setProvenance("syn375811", name="test")

        .. warning:: may be obsolet. Look into original synapseclient instead
        """
        activity = synapseclient.Activity(name=name)
        super(SynapseClient, self).setProvenance(entity, activity)

    def __getEvaluation(self, eid):
        """Returns an evaluation

        obsolet available in synapseclient now"""
        e = self.restGET("/evaluation/%s" % eid)
        return e

    def __createWiki(self, owner, title, markdown, owner_type=None):
        """

        :param owner: the owner object (entity, competition, evaluation) 
            with which the new wiki page will be associated
        :param markdown: the contents of the wiki page in markdown
        :param owner_type: if not provided, the client tries to figure out the type.
            e.g., Works for evaluation 
        """
        try:
            res = self._createWiki(owner, title, markdown, owner_type=owner_type)
            return res
        except Exception, e:
            print("Could not create the wiki. Exists already ?")
            raise Exception

    def __getWiki_TC(self, owner, owner_type):
        """Returns wiki given owner and owner_type

        """
        res = self.restGET("/%s/%s/wiki" % (owner_type, owner['id']))
        return res

    def __createWikiChild(self, owner, title):
        # retrieve the main wiki
        wiki = self.getWiki_TC(owner, "evaluation")
        wikiURI = "/evaluation/%s/wiki" % owner['id']

        print(wikiURI)
        wikiChild = synapseclient.Wiki(title=title, markdown="", parentWikiId=wiki['id'])
        res = self.restPOST(wikiURI, wikiChild.getjson())

        return res

    def getMyProfile(self):
        """Returns user profile"""
        return self.restGET("/userProfile")

    def __joinEvaluation(self, evalId):
        """Join an existing evaluation

        :param evalIe: a valid evaluation identifier
        """
        prof =  self.getMyProfile()
        try:
            self.restPOST("/evaluation/%s/participant/%s" % (evalId, prof['ownerId']), list())
        except Exception, e:
            print("Joining evaluation failed. Maybe you've already joined.")

    def json(self, data):
        """Transform relevant object into json object"""
        import json
        data = json.dumps(data)
        return data


def __time2datetime(timedate):
    """not used anymore ???"""
    import datetime

    date = [int(x) for x in timedate.split("T")[0].split("-")]
    time = timedate.split("T")[1].split(":")
    print time
    time[2] = time[2].split(".")[0]
    time = [int(x) for x in time]
    this = datetime.datetime(date[0], date[1], date[2], time[0], time[1], time[2])
    return this
    





from BeautifulSoup import BeautifulSoup
import urllib2




class GetTeamMail(object):
    def __init__(self, lowercase=False, url=None, filename=None):
        """Manipulate the team/mail pairs from Dream challenge log file

        This function reads an URL from the Dream website (
        http://www.the-dream-project.org/file_downloads/1/feeds?attach=page_1) that
        returns a CSV document. This document is parsed to get pair of email/team names.

        This web page is a log that is not always up-to-date. A new and more
        complete page is available but cannot be accessed with scripting tools
        at the moment. So, you can also provide a file that
        contains two columns : team name and emails (see :meth:`read_mails_from_file`
        and the example below).

        The instance contains the team names, and values containing their emails.

        :param lowercase: if true (default) return the team names as lower case
        :param url: overwrite the default url
        :param filename: if provided, the url is replaced by the contents of this file

        :return: a dictionary of team names versus emails

        :Example:

        The following example retrieve team/mail pairs from the default url.

            >>> import dreamtools
            >>> from dreamtools import mailing
            >>> mails = mailing.GetTeamMail()
            >>> print mails

        You can read a list in a file instead:

            >>> import dreamtools
            >>> from dreamtools import mailing
            >>> mails = mailing.GetTeamMail(filename='share/data/list.txt')
            >>> print mails


        """
        if url == None:
            self.url = 'http://www.the-dream-project.org/file_downloads/1/feeds?attach=page_1'
        else:
            self.url = url


        if filename:
            self._read_mails_from_file(filename)
        else:
            # read the url page
            u = urllib2.urlopen(self.url)
            source = u.read()
            soup = BeautifulSoup(source)

            mails = {}

            data = str(soup).split('\n')
            # remove the header
            del(data[0])

            # scan all lines
            for line in data:
                fields = line.split(',')
                if len(fields) != 8:
                    pass
                else:
                    if lowercase:
                        team = str(fields[6].lower().replace('"', ''))
                    else:
                        team = str(fields[6].replace('"', ''))
                    mail = str(fields[5].replace('"', ''))
                    mails[team] = mail
            self.data = mails

    def getTeamsGivenMails(self, emails_list):
        """

        :Usage:

            >>> m = GetTeamMail(file='test')
            >>> teamNames = m.getTeamsGivenMails(['test@yahoo.fr'])

        """
        return [team for team, email in self.data.iteritems() if email in emails_list]

    def _read_mails_from_file(self, filename):
        """Fill the team/mail with a CSV file instead of url

        Expect a 2-columns file to contain team names in the first
        column and email in the second one

        """
        f = open(filename, 'r')
        data = f.readlines()
        f.close()
        self.data = {}
        for item in data:
            team, mail = item.split()
            self.data[team] =  mail.rstrip('\n')

    def __str__(self):

        str_ = "Found teamName / mails -------------------------------\n"
        for teamName in self.data.keys():
            try:
                str_ += '    ' + teamName +" "+ self.data[teamName] +'\n'
            except Exception,e:
                str_ += self.lookfor(teamName)
                print e

        str_ += "\n\nMailing list is ------------------------\n\n"
        for teamName in self.data.keys():
            try:
                str_ +=  self.data[teamName] + ', '
            except Exception,e:
                str_ += self.lookfor(teamName) + '\n'
                print e
        return str_

    def lookfor(self, team):
        """Look for a team name in the data attributes

        >>> m = mailing.GetTeamMail(filename='./share/data/list.txt')
        >>> m.lookfor('fly')

        If found, print the team name and emails and return the team name provided.
        If not found, try to find an equivalent name (comparing lower case only).

        """
        str_ = ''
        for this in self.data.keys():
            if this == team:
                print this, self.data[team]
                return this
            elif team.lower() in this.lower():
                str_ = team + ' Not found. Did you mean '+ this + '?'
                break
        if str_ == '':
            str_ = 'Team ' + team + ' not found.'
        return str_







# -*- python -*-
# -*- coding: utf-8 -*-
#
#  This file is part of dreamtools software
#
#  Copyright (c) 2013-2015 - EBI-EMBL
#
#  File author(s): Thomas Cokelaer <cokelaer@ebi.ac.uk>
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: http://github.com/dreamtools
#
##############################################################################

"""
Created on Fri Mar 21 11:51:26 2014

@author: cokelaer
"""

cellLines = ["BT20", "MCF7", "UACC812", "BT549"]
ligands = ["EGF", "HGF", "FGF1", "IGF1", "Insulin", "NRG1", "PBS", "Serum"]

valid_ligands_final = {
                   'UACC812': ['PBS', 'Serum', 'NRG1', 'Insulin', 'HGF', 'EGF', 'FGF1',  'IGF1'],
                      'BT20': ['IGF1', 'PBS', 'Serum', 'NRG1', 'HGF', 'EGF', 'FGF1'],
                      'BT549': ['IGF1', 'PBS', 'Serum', 'Insulin', 'HGF', 'EGF', 'FGF1'],
                      'MCF7': ['IGF1', 'PBS', 'Serum', 'NRG1', 'Insulin',  'HGF', 'EGF', 'FGF1']
                      }


metadata = {
    'sc1a': {"true_synapse_id": "syn1971278"},
    'sc1b': {},
    'sc2a': {},
    'sc2b': {},
}



from dreamtools.dream8.D8C1 import aggregation




def test_sc2a():
    a = aggregation.SC2A_aggregation()
    a.load_submissions()
    aucs = a.plot_aggr_best_score()
    assert aucs == [0.48434056686051674, 0.45297349456396707]
    aucs = a.plot_aggr_random(N=2, Nmax=2)


def test_sc2b():
    b = aggregation.SC2B_aggregation()
    b.load_submissions()
    aucs = b.plot_aggr_best_score()
    assert aucs == [0.24554824475398279, 0.25530788468348198]    
    aucs = b.plot_aggr_random(N=2, Nmax=2)

def test_sc1a():
    b = aggregation.SC1A_aggregation()
    b.load_submissions()
    aucs = b.plot_aggr_best_score(M=2)
    assert aucs['aggregation'] ==  [0.78202740472227761, 0.79045976573183718]
    aucs = b.plot_aggr_random(2,2)



def test_sc1b():
    b = aggregation.SC1B_aggregation()
    b.load_submissions()
    aucs = b.plot_aggr_best_score(M=2)
    assert aucs == [0.76422010245539662, 0.73489666136724974]
    aucs = b.plot_aggr_random(2,2)




# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages
from Cython.Build import cythonize


_MAJOR               = 0
_MINOR               = 1
_MICRO               = 0
version              = '%d.%d.%d' % (_MAJOR, _MINOR, _MICRO)
release              = '%d.%d' % (_MAJOR, _MINOR)


## compile cython code
def post_process():
    cwd = os.getcwd()
    os.chdir('dreamtools/dream8/D8C1')
    os.system('python setup.py build_ext --inplace')
    os.chdir(cwd)


metainfo = {
    'authors': {
        'Cokelaer':('Thomas Cokelaer','cokelaer@gmail.com'),
        'Eduati': ('Federica Eduati', 'eduati@ebi.ac.uk')
        },
    'version': version,
    'license' : 'BSD',
    'download_url' : ['http://pypi.python.org/pypi/dreamtools'],
    'url' : ['https://github.com/dreamtools/dreamtools'],
    'description':'Scoring functions for the dream / sage challenges' ,
    'platforms' : ['Linux', 'Unix', 'MacOsX', 'Windows'],
    'keywords' : ['scoring', 'dream', 'roc', 'leaderboard'],
    'classifiers' : [
          'Development Status :: 1 - Planning',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'License :: OSI Approved :: BSD License',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Scientific/Engineering :: Bio-Informatics',
          'Topic :: Scientific/Engineering :: Information Analysis',
          'Topic :: Scientific/Engineering :: Mathematics',
          'Topic :: Scientific/Engineering :: Physics']
    }



with open('README.rst') as f:
    readme = f.read()



setup(
    name             = 'dreamtools',
    version          = version,
    maintainer       = metainfo['authors']['Cokelaer'][0],
    maintainer_email = metainfo['authors']['Cokelaer'][1],
    author           = metainfo['authors']['Cokelaer'][0],
    author_email     = metainfo['authors']['Cokelaer'][1],
    long_description = readme,
    keywords         = metainfo['keywords'],
    description = metainfo['description'],
    license          = metainfo['license'],
    platforms        = metainfo['platforms'],
    url              = metainfo['url'],
    download_url     = metainfo['download_url'],
    classifiers      = metainfo['classifiers'],

    zip_safe=False,
    packages = find_packages(),
    # package installation

    include_package_data = True,
    # (you can provide an exclusion dictionary named exclude_package_data to remove parasites).
    # alternatively to global inclusion, list the file to include
    package_data = {'' : ['*.txt', '*.so', '*.zip', '*.csv', '*.ini', '*.R'],},


    # distutils in rtools.package
    #install_requires = [ 'pandas', 'bioservices', 'colormap>=0.9.3'],
    install_requires = ['numpy', 'matplotlib', 'pandas', 'appdirs',
        'easydev>=0.8.3', 'fitter',
        'synapseclient', 'tabulate', 'cython'],

    ext_modules = cythonize("dreamtools/dream8/D8C1/*.pyx"),


    entry_points = {
        'console_scripts': [
            'dreamtools-scoring=dreamtools.core.scoring:scoring',
        ]
    },
)

#using cythonize command gets the compiled cython code into the .egg
#post_process()



from dreamtools.dream7.D7C1 import scoring
import os
import nose

def test_scoring_model1():
    s = scoring.D7C1()

    ################## model1 parameters:
    # best performer
    filename = s._path2data + os.sep + 'templates' + \
            os.sep + 'model1_parameters_alphabeta.txt'
    score = s.score_model1_parameters(filename)
    nose.tools.assert_almost_equal(score,0.02286755)
    # gold standard
    filename = s._path2data + os.sep + 'goldstandard' + \
            os.sep + 'model1_parameters_answer.txt'
    score = s.score_model1_parameters(filename)
    assert score == 0


def test_scoring_topology_leaderboard():
    s = scoring.D7C1()
    try:
        s.compute_score_topology()
        assert all(s.scores['topo2'].scores == [2,3,4,4,5,6,7,8,8,8,9,12])
    except:
        # needs all submissions
        pass

def test_scoring_topology():
    s = scoring.D7C1()
    filename = s._path2data + os.sep + 'templates' + os.sep 
    filename += 'network_topology_alphabeta.txt'
    score = s.score_topology(filename)
    assert score == 4

def test_scoring_parameters():
    s = scoring.D7C1()
    filename = s._path2data + os.sep + 'templates' + os.sep 
    filename += 'model1_parameters_alphabeta.txt'
    score = s.score_model1_parameters(filename)
    nose.tools.assert_almost_equal(score, 0.02286755501)

def test_scoring_timecourse():
    s = scoring.D7C1()
    filename = s._path2data + os.sep + 'templates' + os.sep 
    filename += 'model1_timecourse_alphabeta.txt'
    score = s.score_model1_timecourse(filename)
    nose.tools.assert_almost_equal(score, 0.002438361267)

def _test_others():
    s = scoring.D7C1()
    df = s.get_null_parameters_model1()





# can be used by all
cellLines = ["BT20", "MCF7", "UACC812", "BT549"]
ligands = ["EGF", "HGF", "FGF1", "IGF1", "Insulin", "NRG1", "PBS", "Serum"]


metadata = {
    'sc1a': {"true_synapse_id": "syn1971278"},
    'sc1b': {},
    'sc2a': {},
    'sc2b': {},
        }


import commons
import scoring
from scoring import *
import hpn
from hpn import *
import submissions

import os
from dreamtools import dreampath
d8c1path = os.sep.join([dreampath, 'dream8', 'D8C1'])

# SC1A network
experimental_data_synapseId = "syn1920412"
experimental_filename = d8c1path + os.sep + 'experimental.zip'



# download data if not found
# syn1920412
